from logging import Formatter, LogRecord, setLogRecordFactory
from logging.config import dictConfig
from typing import Any, Dict

from . import helper


class LogExtraFactory(LogRecord):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        request_id = helper.request_id_ctx.get() or "N/A"

        self.__dict__["request_id"] = request_id


def init_logger(config: Dict[str, Any]):
    dictConfig(config)
    setLogRecordFactory(LogExtraFactory)


class RequestIdFormatter(Formatter):
    def format(self, record):
        if not hasattr(record, "request_id"):
            record.request_id = "N/A"
        return super().format(record)



from .helper import RequestIdCtx, request_id_ctx, set_request_id
from .logger import LogExtraFactory, RequestIdFormatter, init_logger

__all__ = [
    "LogExtraFactory",
    "RequestIdFormatter",
    "RequestIdCtx",
    "init_logger",
    "request_id_ctx",
    "set_request_id",
]



#!/usr/bin/env python
import pathlib
from importlib.machinery import SourceFileLoader

from setuptools import find_packages, setup

version = SourceFileLoader("version", "request_id_helper/version.py").load_module()


_ROOT = pathlib.Path(__file__).parent

with open(str(_ROOT / "README.md")) as f:
    readme = f.read()

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.12",
    "Programming Language :: Python :: 3.13",
    "Operating System :: POSIX",
    "Topic :: Software Development",
    "Topic :: Software Development :: Libraries",
    "Environment :: Console",
    "Intended Audience :: Developers",
]

setup(
    name="request-id-helper",
    version=str(version.VERSION),
    license="Apache License, Version 2.0",
    description="Helper to add request id in logger and context",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Pavel Liashkov",
    author_email="pavel.liashkov@protonmail.com",
    maintainer="Pavel Liashkov",
    maintainer_email="pavel.liashkov@protonmail.com",
    download_url="https://pypi.python.org/pypi/request-id-helper",
    url="https://github.com/bigbag/request-id-helper",
    platforms=["POSIX"],
    classifiers=CLASSIFIERS,
    python_requires=">=3.7",
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite="",
)



import asyncio
import uuid
from functools import wraps
from typing import Callable

from . import context


class RequestIdCtx(context.ContextStorage):
    def __init__(self, id_generator: Callable[..., str] = lambda: str(uuid.uuid4())):
        super().__init__()
        self.id_generator = id_generator

    def create(self) -> str:
        value = self.id_generator()
        self.set(value)
        return value


request_id_ctx = RequestIdCtx()


def set_request_id():
    def decorator(fn):
        if asyncio.iscoroutinefunction(fn):

            @wraps(fn)
            async def wrapper(*args, **kwargs):
                request_id_ctx.create()
                return await fn(*args, **kwargs)

        else:

            @wraps(fn)
            def wrapper(*args, **kwargs):
                request_id_ctx.create()
                return fn(*args, **kwargs)

        return wrapper

    return decorator
